import 'dart:collection';
import 'dart:developer';

import 'package:auto_size_text/auto_size_text.dart';
import 'package:flutter/cupertino.dart';

import '/flutter_flow/flutter_flow_theme.dart';
import '/flutter_flow/flutter_flow_util.dart';
import 'package:cached_network_image/cached_network_image.dart';
import 'package:flutter/material.dart';
import 'package:google_fonts/google_fonts.dart';
import 'package:provider/provider.dart';
import 'card_item_template_component_model.dart';
export 'card_item_template_component_model.dart';

///

class CardItemTemplateComponentWidget extends StatefulWidget {
  const CardItemTemplateComponentWidget({
    super.key,
    required this.labelsHashMap,
    required this.statusHashMap,
    required this.imagePath,
    required this.onTapCardItemAction,
  });

  final Map<String, String>? labelsHashMap;
  final List<Map<String, Color>?> statusHashMap;
  final String? imagePath;
  final Future Function()? onTapCardItemAction;

  @override
  State<CardItemTemplateComponentWidget> createState() => _CardItemTemplateComponentWidgetState();
}

class _CardItemTemplateComponentWidgetState extends State<CardItemTemplateComponentWidget> {
  late CardItemTemplateComponentModel _model;
  LinkedHashMap<String, String> get labelsLinkedHashMap => LinkedHashMap.from(widget.labelsHashMap ?? {});
  List<LinkedHashMap<String, Color>> get statusLinkedHashMap => widget.statusHashMap.map((map) => LinkedHashMap<String, Color>.from(map ?? {})).toList();

  @override
  void setState(VoidCallback callback) {
    super.setState(callback);
    _model.onUpdate();
  }

  @override
  void initState() {
    super.initState();
    _model = createModel(context, () => CardItemTemplateComponentModel());
  }

  @override
  void dispose() {
    super.dispose();
    _model.maybeDispose();
  }

  String truncate(int cutoff, String text) {
    return (text.length <= cutoff) ? text : '${text.substring(0, cutoff)}...';
  }

  List<Widget> _generateLabels() {
    List<Widget> labels = [];
    double width = MediaQuery.of(context).size.width;

    for (var key in labelsLinkedHashMap.keys) {
        String text = labelsLinkedHashMap[key].toString();

        Widget widget = Row(
          children: [
            Text(
              key,
              style: FlutterFlowTheme.of(context).bodyMedium.override(
                fontFamily: FlutterFlowTheme.of(context).bodyMediumFamily,
                letterSpacing: 0.0,
                fontWeight: FontWeight.bold,
                useGoogleFonts: GoogleFonts.asMap().containsKey(FlutterFlowTheme.of(context).bodyMediumFamily),
                color: FlutterFlowTheme.of(context).primaryText,
              ),
            ),
            const SizedBox(width: 2.5),
            SizedBox(
              width: 225,
              child: Text(
                text,
                overflow: TextOverflow.ellipsis,
                style: FlutterFlowTheme.of(context).bodyMedium.override(
                  fontFamily: FlutterFlowTheme.of(context).bodyMediumFamily,
                  fontSize: 12.5,
                  letterSpacing: 0.0,
                  fontWeight: FontWeight.bold,
                  useGoogleFonts: GoogleFonts.asMap().containsKey(FlutterFlowTheme.of(context).bodyMediumFamily),
                ),
              ),
            )
          ],
        );

        labels.add(widget);
    }

    return labels;
  }

  Widget _generateImage() {
    return Container(
      width: 100,
      height: 100,
      decoration: BoxDecoration(
        color: FlutterFlowTheme.of(context).primaryBackground,
      ),
      child: ClipRRect(
        borderRadius: BorderRadius.circular(20),
        child: CachedNetworkImage(
          fadeInDuration: const Duration(milliseconds: 500),
          fadeOutDuration: const Duration(milliseconds: 500),
          imageUrl: widget.imagePath ?? '',
          fit: BoxFit.cover,
        ),
      ),
    );
  }

  List<Widget> _generateStatus() {
    List<Widget> status = [];

    statusLinkedHashMap.expand((statusLinked) {
      statusLinked.forEach((text, color) {
        Widget widget = Container(
          padding: const EdgeInsets.all(5),
          decoration: BoxDecoration(
            color: color,
            borderRadius: BorderRadius.circular(5)
          ),
          child: Text(
            text,
            overflow: TextOverflow.ellipsis,
            style: TextStyle(
              color: FlutterFlowTheme.of(context).info,
              fontSize: 12,
              fontWeight: FontWeight.bold
            ),
          ),
        );

        status.add(widget);
      });

      return [];
    }).toList();
    
    return status;
  }

  Widget _smallScreen() {
    return Column(
      mainAxisSize: MainAxisSize.max,
      mainAxisAlignment: MainAxisAlignment.spaceBetween,
      children: [
        _generateImage(),
        const SizedBox(height: 5),
        Column(
            mainAxisSize: MainAxisSize.max,
            crossAxisAlignment: CrossAxisAlignment.start,
            children: [
              ..._generateLabels(),
              const SizedBox(height: 5),
              Wrap(
                spacing: 8,
                runSpacing: 8,
                children: _generateStatus(),
              )
            ]
        ),
      ],
    );
  }

  Widget _largeScreen() {
    return Row(
      mainAxisSize: MainAxisSize.max,
      mainAxisAlignment: MainAxisAlignment.spaceBetween,
      children: [
        Column(
            mainAxisSize: MainAxisSize.max,
            crossAxisAlignment: CrossAxisAlignment.start,
            children: [
              ..._generateLabels(),
              const SizedBox(height: 5),
              Wrap(
                spacing: 8,
                runSpacing: 8,
                children: _generateStatus(),
              )
            ]
        ),
        _generateImage()
      ],
    );
  }

  @override
  Widget build(BuildContext context) {
    double screenWidth = MediaQuery.of(context).size.width;
    double screenHeight = MediaQuery.of(context).size.height;

    context.watch<AppState>();

    return InkWell(
      splashColor: Colors.transparent,
      focusColor: Colors.transparent,
      hoverColor: Colors.transparent,
      highlightColor: Colors.transparent,
      onTap: () async => await widget.onTapCardItemAction?.call(),
      child: Padding(
        padding: const EdgeInsets.all(8),
        child: Card(
          clipBehavior: Clip.antiAliasWithSaveLayer,
          color: FlutterFlowTheme.of(context).primaryBackground,
          elevation: 5.0,
          shape: RoundedRectangleBorder(
            borderRadius: BorderRadius.circular(8.0),
          ),
          child: Container(
            width: double.infinity,
            decoration: BoxDecoration(
              color: FlutterFlowTheme.of(context).primaryBackground,
            ),
            padding: const EdgeInsets.all(8),
            child: Column(
              mainAxisSize: MainAxisSize.max,
              crossAxisAlignment: CrossAxisAlignment.start,
              children: [
                screenWidth > 360 ?  _largeScreen() : _smallScreen(),
              ],
            ),
          ),
        ),
      ),
    );
  }
}
